/* Copyright (C) 2008 Patrick Galbraith, Brian Aker

  See COPYING file found with distribution for license.

*/

#include <mysql.h>
#include <string.h>

#include <stdio.h>

#include "common.h"

my_bool memc_increment_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
long long memc_increment(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void memc_increment_deinit(UDF_INIT *initid);

my_bool memc_increment_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
{
  unsigned int x;
  memcached_return rc;
  memc_function_st *container;

  /* this is how to fail */
  if (args->arg_count > 2 || args->arg_count < 1)
  {
    strncpy(message, "Only one or two arguments are allowed (key, offset).", MYSQL_ERRMSG_SIZE);
    return 1;
  }
  /* Force all arguments into strings, you probably want to do this */
  for (x= 0; x < args->arg_count; x++)
    args->arg_type[x]= STRING_RESULT;

  container= (memc_function_st *)malloc(sizeof(memc_function_st));

  /* Init the memcached_st we will use for this pass */
  rc= memc_get_servers(&container->memc);
  memcached_result_create(&container->memc, &container->results);

  initid->ptr= (char *)container;

  return 0;
}

long long memc_increment(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
{
  memcached_return rc;
  unsigned int value;
  unsigned int offset;
  memcached_st *memc= (memcached_st *)initid->ptr;
  offset= args->arg_count == 2 ? (unsigned int)atoi(args->args[1]) : 1;

  rc= memcached_increment(memc,
                    args->args[0], (size_t)args->lengths[0],
                    offset,
                    &value);

  return ((long long)value);
}

void memc_increment_deinit(UDF_INIT *initid)
{
  /* if we allocated initid->ptr, free it here */
  memc_function_st *container= (memc_function_st *)initid->ptr;

  memcached_result_free(&container->results);
  memcached_free(&container->memc);
  free(container);
}
