/* Copyright (C) 2008 Patrick Galbraith, Brian Aker

  See COPYING file found with distribution for license.

*/

#include <mysql.h>
#include <string.h>
#include <pthread.h>

#include <stdio.h>

#include "common.h"

pthread_mutex_t memc_servers_mutex= PTHREAD_MUTEX_INITIALIZER;

/*
  select memc_servers_set("hostname:23,hostname2:44");
*/

static memcached_st *master_memc= NULL;


my_bool memc_servers_set_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
char *memc_servers_set(UDF_INIT *initid, UDF_ARGS *args, char *result, unsigned long *length, char *is_null, char *error);
/*void memc_servers_set_deinit(UDF_INIT *initid);*/

my_bool memc_servers_behavior_set_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
char *memc_servers_behavior_set(UDF_INIT *initid, UDF_ARGS *args, char *result,
                                unsigned long *length, char *is_null, char *error);
/*void memc_servers_behavior_set_deinit(UDF_INIT *initid);*/

my_bool memc_servers_set_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
{
  unsigned int x;
  memcached_return rc;

  /* this is how to fail */
  if (args->arg_count != 1)
  {
    strncpy(message, "String of memcached servers.", MYSQL_ERRMSG_SIZE);
    return 1;
  }

  args->arg_type[0]= STRING_RESULT;

  /* initid->ptr keeps state for between memc_get_init and memc_get_deinit */
  initid->ptr= NULL;

  return 0;
}

/*
  memc_servers_set
  get cached object, takes hash-key arg
*/
char *memc_servers_set(UDF_INIT *initid, UDF_ARGS *args, char *result,
                  unsigned long *length, char *is_null, char *error)
{
  pthread_mutex_lock(&memc_servers_mutex);
  if (master_memc == NULL)
    master_memc= memcached_create(NULL);

  servers= memcached_servers_parse(args->args[0]);

  pthread_mutex_unlock(&memc_servers_mutex);

  return (0);
}



/* memc_servers_behavior_set_init */
my_bool memc_servers_behavior_set_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
{
  unsigned int x;
  memcached_return rc;
  memc_function_st *container;

  /* Fail if not two args */
  if (args->arg_count != 2)
  {
    strncpy(message, "Two argument for memc_servers_behavior_set - behavior type, value.\n", MYSQL_ERRMSG_SIZE);
    return 1;
  }

  for (x= 0; x < args->arg_count; x++)
    args->arg_type[x]= STRING_RESULT;

  /*
    Brian: Ok, so I clone here, but do I also need to lock ?
  */
  pthread_mutex_lock(&memc_servers_mutex);
  rc= memcached_clone(container->memc, master_memc);
  pthread_mutex_unlock(&memc_servers_mutex);

  /* initid->ptr keeps state for between memc_get_init and memc_get_deinit */
  initid->ptr= (char *)container;

  return 0;
}

/* memc_servers_behavior_set */
char *memc_servers_behavior_set(UDF_INIT *initid, UDF_ARGS *args, char *result,
                                unsigned long *length, char *is_null, char *error)
{
  /*
    Brian: The user will be required to know the behavior values, right?.
    Should we allow lower-case?
  */
  memcached_behavior behavior;
  memcached_return rc;

  /* 
    Get a lock patrick, and modify the behavior of the master_memc 
    You will need to convert "string" from param to one of the enum
    that changes memcached_st behavior.

    See memcached_behavior_set for examples.
  */
 
  if (! strcmp(args->args[0], "MEMCACHED_BEHAVIOR_SUPPORT_CAS"))
    behavior= MEMCACHED_BEHAVIOR_SUPPORT_CAS;
  else if (! strcmp(args->args[0], "MEMCACHED_BEHAVIOR_NO_BLOCK"))
    behavior= MEMCACHED_BEHAVIOR_NO_BLOCK;
  else if (! strcmp(args->args[0], "MEMCACHED_BEHAVIOR_TCP_NODELAY"))
    behavior= MEMCACHED_BEHAVIOR_TCP_NODELAY;
  else if (! strcmp(args->args[0], "MEMCACHED_BEHAVIOR_HASH"))
    behavior= MEMCACHED_BEHAVIOR_HASH;
  else if (! strcmp(args->args[0], "MEMCACHED_BEHAVIOR_CACHE_LOOKUPS"))
    behavior= MEMCACHED_BEHAVIOR_CACHE_LOOKUPS;
  else if (! strcmp(args->args[0], "MEMCACHED_BEHAVIOR_KETAMA"))
    behavior= MEMCACHED_BEHAVIOR_KETAMA;
  else if (! strcmp(args->args[0], "MEMCACHED_BEHAVIOR_SOCKET_SEND_SIZE"))
    behavior= MEMCACHED_BEHAVIOR_SOCKET_SEND_SIZE;
  else if (! strcmp(args->args[0], "MEMCACHED_BEHAVIOR_SOCKET_RECV_SIZE"))
    behavior= MEMCACHED_BEHAVIOR_SOCKET_RECV_SIZE;


  memc_function_st *container= (memc_function_st *)initid->ptr;

  pthread_mutex_lock(&memc_servers_mutex);
  /* Brian: can I just cast args->[1] as a void* ? */
  rc= memcached_behavior_set(container->memc, (void *)args->[1], NULL );
  pthread_mutex_unlock(&memc_servers_mutex);
}

/*
  Brian: what do you want to do with this? Is this another UDF?
*/
char *memc_servers_reset(UDF_INIT *initid, UDF_ARGS *args, char *result,
                                unsigned long *length, char *is_null, char *error)
{
  /* Dump all of host names - what/why is this? */
  /* host_reset(&new_host_list[count-1], NULL, 0, MEMCACHED_CONNECTION_UNKNOWN); */
}


/*
   Brian: Is this a helper function for memc_servers_reset?
*/
memcached_return memc_get_servers(memcached_st *memc)
{
  memcached_return rc;

  pthread_mutex_lock(&memc_servers_mutex);
  rc= memcached_clone(container->memc, master_memc);
  pthread_mutex_unlock(&memc_servers_mutex);

  return (unsigned int)rc;
}
